@@ -7,7 +7,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2008 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -23,21 +23,58 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: common.c 1193 2008-06-21 20:37:58Z ngarofil $ */
+ * vim: ts=4 sw=4 noet ai cindent syntax=c
+ *
+ */
 
+#include "config.h"
 #include "conky.h"
+#include "fs.h"
+#include "logging.h"
 #include <ctype.h>
 #include <errno.h>
 #include <sys/time.h>
+#include <sys/ioctl.h>
+#include <net/if.h>
+#include <netinet/in.h>
 #include <pthread.h>
+#include <unistd.h>
+#include "diskio.h"
+#include <fcntl.h>
+
+/* check for OS and include appropriate headers */
+#if defined(__linux__)
+#include "linux.h"
+#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#include "freebsd.h"
+#elif defined(__OpenBSD__)
+#include "openbsd.h"
+#endif
+
+/* OS specific prototypes to be implemented by linux.c & Co. */
+void update_entropy(void);
+
+/* folds a string over top of itself, like so:
+ *
+ * if start is "blah", and you call it with count = 1, the result will be "lah"
+ */
+void strfold(char *start, int count)
+{
+	char *curplace;
+	for (curplace = start + count; *curplace != 0; curplace++) {
+		*(curplace - count) = *curplace;
+	}
+	*(curplace - count) = 0;
+}
 
 #ifndef HAVE_STRNDUP
 // use our own strndup() if it's not available
 char *strndup(const char *s, size_t n)
 {
-	if (strlen(s) + 1 > n) {
-		char *ret = malloc(n);
+	if (strlen(s) > n) {
+		char *ret = malloc(n + 1);
 		strncpy(ret, s, n);
+		ret[n] = 0;
 		return ret;
 	} else {
 		return strdup(s);
@@ -58,9 +95,53 @@ double get_time(void)
 	return tv.tv_sec + (tv.tv_usec / 1000000.0);
 }
 
+/* Converts '~/...' paths to '/home/blah/...' assumes that 'dest' is at least
+ * DEFAULT_TEXT_BUFFER_SIZE.  It's similar to variable_substitute, except only
+ * cheques for $HOME and ~/ in path */
+void to_real_path(char *dest, const char *source)
+{
+	char tmp[DEFAULT_TEXT_BUFFER_SIZE];
+	if (sscanf(source, "~/%s", tmp) || sscanf(source, "$HOME/%s", tmp)) {
+		char *homedir = getenv("HOME");
+		if (homedir) {
+			snprintf(dest, DEFAULT_TEXT_BUFFER_SIZE, "%s/%s", homedir, tmp);
+		} else {
+			ERR("$HOME environment variable doesn't exist");
+			strncpy(dest, source, DEFAULT_TEXT_BUFFER_SIZE);
+		}
+	} else if (dest != source) {	//see changelog 2009-06-29 if you doubt that this check is necessary 
+		strncpy(dest, source, DEFAULT_TEXT_BUFFER_SIZE);
+	}
+}
+
+int open_fifo(const char *file, int *reported)
+{
+	char path[DEFAULT_TEXT_BUFFER_SIZE];
+	int fd = 0;
+
+	to_real_path(path, file);
+	fd = open(file, O_RDONLY | O_NONBLOCK);
+
+	if (fd == -1) {
+		if (!reported || *reported == 0) {
+			ERR("can't open %s: %s", file, strerror(errno));
+			if (reported) {
+				*reported = 1;
+			}
+		}
+		return -1;
+	}
+
+	return fd;
+}
+
 FILE *open_file(const char *file, int *reported)
 {
-	FILE *fp = fopen(file, "r");
+	char path[DEFAULT_TEXT_BUFFER_SIZE];
+	FILE *fp = 0;
+
+	to_real_path(path, file);
+	fp = fopen(file, "r");
 
 	if (!fp) {
 		if (!reported || *reported == 0) {
@@ -69,7 +150,7 @@ FILE *open_file(const char *file, int *reported)
 				*reported = 1;
 			}
 		}
-		return 0;
+		return NULL;
 	}
 
 	return fp;
@@ -135,7 +216,7 @@ void variable_substitute(const char *s, char *dest, unsigned int n)
 
 static struct net_stat netstats[16];
 
-struct net_stat *get_net_stat(const char *dev)
+struct net_stat *get_net_stat(const char *dev, void *free_at_crash1, void *free_at_crash2)
 {
 	unsigned int i;
 
@@ -151,24 +232,71 @@ struct net_stat *get_net_stat(const char *dev)
 	}
 
 	/* wasn't found? add it */
-	if (i == 16) {
-		for (i = 0; i < 16; i++) {
-			if (netstats[i].dev == 0) {
-				netstats[i].dev = strndup(dev, text_buffer_size);
-				return &netstats[i];
-			}
+	for (i = 0; i < 16; i++) {
+		if (netstats[i].dev == 0) {
+			netstats[i].dev = strndup(dev, text_buffer_size);
+			return &netstats[i];
 		}
 	}
 
-	CRIT_ERR("too many interfaces used (limit is 16)");
+	CRIT_ERR(free_at_crash1, free_at_crash2, "too many interfaces used (limit is 16)");
 	return 0;
 }
 
 void clear_net_stats(void)
 {
+	int i;
+	for (i = 0; i < 16; i++) {
+		if (netstats[i].dev) {
+			free(netstats[i].dev);
+		}
+	}
 	memset(netstats, 0, sizeof(netstats));
 }
 
+/* We should check if this is ok with OpenBSD and NetBSD as well. */
+int interface_up(const char *dev)
+{
+	int fd;
+	struct ifreq ifr;
+
+	if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) < 0) {
+		CRIT_ERR(NULL, NULL, "could not create sockfd");
+		return 0;
+	}
+	strncpy(ifr.ifr_name, dev, IFNAMSIZ);
+	if (ioctl(fd, SIOCGIFFLAGS, &ifr)) {
+		/* if device does not exist, treat like not up */
+		if (errno != ENODEV && errno != ENXIO)
+			perror("SIOCGIFFLAGS");
+		goto END_FALSE;
+	}
+
+	if (!(ifr.ifr_flags & IFF_UP)) /* iface is not up */
+		goto END_FALSE;
+	if (ifup_strictness == IFUP_UP)
+		goto END_TRUE;
+
+	if (!(ifr.ifr_flags & IFF_RUNNING))
+		goto END_FALSE;
+	if (ifup_strictness == IFUP_LINK)
+		goto END_TRUE;
+
+	if (ioctl(fd, SIOCGIFADDR, &ifr)) {
+		perror("SIOCGIFADDR");
+		goto END_FALSE;
+	}
+	if (((struct sockaddr_in *)&(ifr.ifr_ifru.ifru_addr))->sin_addr.s_addr)
+		goto END_TRUE;
+
+END_FALSE:
+	close(fd);
+	return 0;
+END_TRUE:
+	close(fd);
+	return 1;
+}
+
 void free_dns_data(void)
 {
 	int i;
@@ -182,7 +310,7 @@ void free_dns_data(void)
 
 //static double last_dns_update;
 
-void update_dns_data(void)
+static void update_dns_data(void)
 {
 	FILE *fp;
 	char line[256];
@@ -257,12 +385,13 @@ static double last_meminfo_update;
 static double last_fs_update;
 
 unsigned long long need_mask;
+int no_buffers;
 
-#define NEED(a) ((need_mask & (1 << a)) && ((info.mask & (1 << a)) == 0))
+#define NEED(a) ((need_mask & (1ULL << a)) && ((info.mask & (1ULL << a)) == 0))
 
 void update_stuff(void)
 {
-	unsigned int i;
+	int i;
 
 	info.mask = 0;
 
@@ -273,6 +402,9 @@ void update_stuff(void)
 	/* clear speeds and up status in case device was removed and doesn't get
 	 * updated */
 
+	#ifdef HAVE_OPENMP
+	#pragma omp parallel for schedule(dynamic,10)
+	#endif /* HAVE_OPENMP */
 	for (i = 0; i < 16; i++) {
 		if (netstats[i].dev) {
 			netstats[i].up = 0;
@@ -315,18 +447,13 @@ void update_stuff(void)
 
 #ifdef MPD
 	if (NEED(INFO_MPD)) {
-		if (!info.mpd.timed_thread) {
-			init_mpd_stats(&info.mpd);
-			info.mpd.timed_thread = timed_thread_create(&update_mpd,
-				(void *) &info.mpd, info.music_player_interval * 1000000);
-			if (!info.mpd.timed_thread) {
-				ERR("Failed to create MPD timed thread");
-			}
-			timed_thread_register(info.mpd.timed_thread, &info.mpd.timed_thread);
-			if (timed_thread_run(info.mpd.timed_thread)) {
-				ERR("Failed to run MPD timed thread");
-			}
-		}
+		update_mpd();
+	}
+#endif
+
+#ifdef MOC
+	if (NEED(INFO_MOC)) {
+		run_moc_thread(info.music_player_interval * 100000);
 	}
 #endif
 
@@ -363,7 +490,7 @@ void update_stuff(void)
 	}
 	
 #ifdef X11
-	if (NEED(INFO_X11)) {
+	if (NEED(INFO_X11) && x_initialised == YES) {
 		update_x11info();
 	}
 #endif
@@ -379,7 +506,7 @@ void update_stuff(void)
 	}
 #ifdef TCP_PORT_MONITOR
 	if (NEED(INFO_TCP_PORT_MONITOR)) {
-		update_tcp_port_monitor_collection(info.p_tcp_port_monitor_collection);
+		tcp_portmon_update();
 	}
 #endif
 	if (NEED(INFO_ENTROPY)) {
@@ -396,9 +523,15 @@ void update_stuff(void)
 	if (NEED(INFO_DNS)) {
 		update_dns_data();
 	}
+#ifdef APCUPSD
+	if (NEED(INFO_APCUPSD)) {
+		update_apcupsd();
+	}
+#endif
 }
 
-int round_to_int(float f)
+/* Ohkie to return negative values for temperatures */
+int round_to_int_temp(float f)
 {
 	if (f >= 0.0) {
 		return (int) (f + 0.5);
@@ -406,3 +539,14 @@ int round_to_int(float f)
 		return (int) (f - 0.5);
 	}
 }
+/* Don't return negative values for cpugraph, bar, gauge, percentage.
+ * Causes unreasonable numbers to show */
+unsigned int round_to_int(float f)
+{
+	if (f >= 0.0) {
+		return (int) (f + 0.5);
+	} else {
+		return 0;
+	}
+}
+
