@@ -8,7 +8,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2008 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
  * (see AUTHORS)
  * All rights reserved.
  *
@@ -24,23 +24,30 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- *  $Id: diskio.h 1090 2008-03-31 04:56:39Z brenden1 $
  */
 
 #ifndef DISKIO_H_
 #define DISKIO_H_
 
 struct diskio_stat {
+	struct diskio_stat *next;
 	char *dev;
-	unsigned int current, current_read, current_write, last, last_read,
-		last_write;
+	double sample[15];
+	double sample_read[15];
+	double sample_write[15];
+	double current;
+	double current_read;
+	double current_write;
+	double last;
+	double last_read;
+	double last_write;
 };
 
-#define MAX_DISKIO_STATS 64
+extern struct diskio_stat stats;
 
-struct diskio_stat *diskio_stats;
-
-struct diskio_stat *prepare_diskio_stat(const char *s);
+struct diskio_stat *prepare_diskio_stat(const char *);
+void update_diskio(void);
 void clear_diskio_stats(void);
+void update_diskio_values(struct diskio_stat *, unsigned int, unsigned int);
 
 #endif /* DISKIO_H_ */
