@@ -29,9 +29,9 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: libmpdclient.c 1154 2008-06-14 18:41:12Z IQgryn $
+ * vim: ts=4 sw=4 noet ai cindent syntax=c
  *
- * */
+ */
 
 #include "conky.h"
 #include "libmpdclient.h"
@@ -128,9 +128,13 @@ static int mpd_connect(mpd_Connection *connection, const char *host, int port,
 	struct addrinfo *res = NULL;
 	struct addrinfo *addrinfo = NULL;
 
-	/* Setup hints */
+	/* Setup hints
+	 *
+	 * XXX: limit address family to PF_INET here.
+	 * MPD does not support IPv6 yet, so if GAI returns
+	 * an IPv6 address, the later connect() will fail. */
 	hints.ai_flags		= AI_ADDRCONFIG;
-	hints.ai_family		= PF_UNSPEC;
+	hints.ai_family		= PF_INET;
 	hints.ai_socktype	= SOCK_STREAM;
 	hints.ai_protocol	= IPPROTO_TCP;
 	hints.ai_addrlen	= 0;
