@@ -7,7 +7,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2008 Markus Meissner
- * Copyright (c) 2005-2008 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -23,7 +23,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: nvidia.h 1231 2008-08-03 13:27:00Z n0-1 $
  */
 
 
@@ -34,11 +33,12 @@
 #include <NVCtrl/NVCtrlLib.h>
 
 typedef enum _QUERY_ID {
-	NV_TEMP = NV_CTRL_GPU_CORE_TEMPERATURE,
-	NV_TEMP_THRESHOLD = NV_CTRL_GPU_CORE_THRESHOLD,
-	NV_GPU_FREQ = NV_CTRL_GPU_CURRENT_CLOCK_FREQS,
-	NV_MEM_FREQ = NV_CTRL_GPU_CURRENT_CLOCK_FREQS,
-	NV_IMAGE_QUALITY = NV_CTRL_IMAGE_SETTINGS
+	NV_TEMP,
+	NV_TEMP_THRESHOLD,
+	NV_TEMP_AMBIENT,
+	NV_GPU_FREQ,
+	NV_MEM_FREQ,
+	NV_IMAGE_QUALITY
 } QUERY_ID;
 
 struct nvidia_s {
