@@ -7,7 +7,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2007 Toni Spets
- * Copyright (c) 2005-2008 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -23,7 +23,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: openbsd.c 1193 2008-06-21 20:37:58Z ngarofil $ */
+ * vim: ts=4 sw=4 noet ai cindent syntax=c
+ *
+ */
 
 #include <sys/dkstat.h>
 #include <sys/param.h>
@@ -56,6 +58,10 @@
 #include <net80211/ieee80211_ioctl.h>
 
 #include "conky.h"
+#include "diskio.h"
+#include "logging.h"
+#include "openbsd.h"
+#include "top.h"
 
 #define	MAXSHOWDEVS		16
 
@@ -180,9 +186,11 @@ void update_meminfo()
 	if ((swapmode(&swap_used, &swap_avail)) >= 0) {
 		info.swapmax = swap_avail;
 		info.swap = swap_used;
+		info.swapfree = swap_avail - swap_used;
 	} else {
 		info.swapmax = 0;
 		info.swap = 0;
+		info.swapfree = 0;
 	}
 }
 
@@ -205,7 +213,7 @@ void update_net_stats()
 	}
 
 	for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
-		ns = get_net_stat((const char *) ifa->ifa_name);
+		ns = get_net_stat((const char *) ifa->ifa_name, NULL, NULL);
 
 		if (ifa->ifa_flags & IFF_UP) {
 			struct ifaddrs *iftmp;
@@ -321,14 +329,14 @@ void get_cpu_count()
 
 	info.cpu_usage = malloc(info.cpu_count * sizeof(float));
 	if (info.cpu_usage == NULL) {
-		CRIT_ERR("malloc");
+		CRIT_ERR(NULL, NULL, "malloc");
 	}
 
 #ifndef OLDCPU
 	assert(fresh == NULL);	/* XXX Is this leaking memory? */
 	/* XXX Where shall I free this? */
 	if (NULL == (fresh = calloc(cpu_count, sizeof(int64_t) * CPUSTATES))) {
-		CRIT_ERR("calloc");
+		CRIT_ERR(NULL, NULL, "calloc");
 	}
 #endif
 }
@@ -598,6 +606,7 @@ char get_freq(char *p_client_buffer, size_t client_buffer_size,
 
 void update_top()
 {
+	kvm_init();
 	proc_find_top(info.cpu, info.memu);
 }
 
@@ -654,6 +663,14 @@ cleanup:
 }
 #endif
 
+void clear_diskio_stats()
+{
+}
+
+struct diskio_stat *prepare_diskio_stat(const char *s)
+{
+}
+
 void update_diskio()
 {
 	return;	/* XXX: implement? hifi: not sure how */
@@ -695,18 +712,22 @@ inline void proc_find_top(struct process **cpu, struct process **mem)
 	struct process *processes;
 	int mib[2];
 
-	int total_pages;
+	u_int total_pages;
+	int64_t usermem;
 	int pagesize = getpagesize();
 
 	/* we get total pages count again to be sure it is up to date */
 	mib[0] = CTL_HW;
-	mib[1] = HW_USERMEM;
-	size_t size = sizeof(total_pages);
+	mib[1] = HW_USERMEM64;
+	size_t size = sizeof(usermem);
 
-	if (sysctl(mib, 2, &total_pages, &size, NULL, 0) == -1) {
-		ERR("error reading nmempages");
+	if (sysctl(mib, 2, &usermem, &size, NULL, 0) == -1) {
+		ERR("error reading usermem");
 	}
 
+	/* translate bytes into page count */
+	total_pages = usermem / pagesize;
+
 	int max_size = sizeof(struct kinfo_proc2);
 
 	p = kvm_getproc2(kd, KERN_PROC_ALL, 0, max_size, &n_processes);
@@ -717,8 +738,8 @@ inline void proc_find_top(struct process **cpu, struct process **mem)
 			processes[j].pid = p[i].p_pid;
 			processes[j].name = strndup(p[i].p_comm, text_buffer_size);
 			processes[j].amount = 100.0 * p[i].p_pctcpu / FSCALE;
-			processes[j].totalmem = (float) (p[i].p_vm_rssize * pagesize /
-				(float) total_pages) * 100.0;
+			processes[j].totalmem = (float) (p[i].p_vm_rssize /
+					(float) total_pages) * 100.0;
 			j++;
 		}
 	}
