@@ -1,7 +1,7 @@
 /* smapi.h:  conky support for IBM Thinkpad smapi
  *
  * Copyright (C) 2007 Phil Sutter <Phil@nwl.cc>
- * 
+ *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -17,12 +17,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA.
  *
- * $Id: smapi.h 1073 2008-03-29 10:13:25Z n0-1 $
- *
  */
 
-#ifndef SMAPI_H
-#define SMAPI_H
+#ifndef _SMAPI_H
+#define _SMAPI_H
 
 int smapi_bat_installed(int);
 
@@ -36,4 +34,4 @@ char *smapi_get_bat_str(int, const char *);
 int smapi_get_bat_int(int, const char *);
 char *smapi_get_bat_val(const char *);
 
-#endif /* SMAPI_H */
+#endif /* _SMAPI_H */
