@@ -8,7 +8,7 @@
  *
  * Copyright (c) 2005 Adi Zaimi, Dan Piponi <dan@tanelorn.demon.co.uk>,
  *					  Dave Clark <clarkd@skynet.ca>
- * Copyright (c) 2005-2008 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -24,7 +24,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: top.c 1187 2008-06-21 14:18:44Z ngarofil $ */
+ * vim: ts=4 sw=4 noet ai cindent syntax=c
+ *
+ */
 
 #include "top.h"
 
@@ -52,6 +54,18 @@ void free_all_processes(void)
 	first_process = NULL;
 }
 
+struct process *get_process_by_name(const char *name)
+{
+	struct process *p = first_process;
+
+	while (p) {
+		if (!strcmp(p->name, name))
+			return p;
+		p = p->next;
+	}
+	return 0;
+}
+
 static struct process *find_process(pid_t pid)
 {
 	struct process *p = first_process;
@@ -87,6 +101,10 @@ static struct process *new_process(int p)
 	process->time_stamp = 0;
 	process->previous_user_time = ULONG_MAX;
 	process->previous_kernel_time = ULONG_MAX;
+#ifdef IOSTATS
+	process->previous_read_bytes = ULLONG_MAX;
+	process->previous_write_bytes = ULLONG_MAX;
+#endif
 	process->counted = 1;
 
 	/* process_find_name(process); */
@@ -113,9 +131,9 @@ static int process_parse_stat(struct process *process)
 	unsigned long kernel_time = 0;
 	int rc;
 	char *r, *q;
-	char deparenthesised_name[BUFFER_LEN];
 	int endl;
 	int nice_val;
+	char *lparen, *rparen;
 
 	snprintf(filename, sizeof(filename), PROCFS_TEMPLATE, process->pid);
 
@@ -135,16 +153,22 @@ static int process_parse_stat(struct process *process)
 	}
 
 	/* Extract cpu times from data in /proc filesystem */
-	rc = sscanf(line, "%*s %s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %lu "
-		"%lu %*s %*s %*s %d %*s %*s %*s %u %u", procname, &process->user_time,
+	lparen = strchr(line, '(');
+	rparen = strrchr(line, ')');
+	if(!lparen || !rparen || rparen < lparen)
+		return 1; // this should not happen
+
+	rc = MIN((unsigned)(rparen - lparen - 1), sizeof(procname) - 1);
+	strncpy(procname, lparen + 1, rc);
+	procname[rc] = '\0';
+	rc = sscanf(rparen + 1, "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %lu "
+		"%lu %*s %*s %*s %d %*s %*s %*s %u %u", &process->user_time,
 		&process->kernel_time, &nice_val, &process->vsize, &process->rss);
 	if (rc < 5) {
 		return 1;
 	}
-	/* Remove parentheses from the process name stored in /proc/ under Linux */
-	r = procname + 1;
 	/* remove any "kdeinit: " */
-	if (r == strstr(r, "kdeinit")) {
+	if (procname == strstr(procname, "kdeinit")) {
 		snprintf(filename, sizeof(filename), PROCFS_CMDLINE_TEMPLATE,
 			process->pid);
 
@@ -166,24 +190,18 @@ static int process_parse_stat(struct process *process)
 			r = (char *) line;
 		}
 
-		q = deparenthesised_name;
+		q = procname;
 		/* stop at space */
 		while (*r && *r != ' ') {
 			*q++ = *r++;
 		}
 		*q = 0;
-	} else {
-		q = deparenthesised_name;
-		while (*r && *r != ')') {
-			*q++ = *r++;
-		}
-		*q = 0;
 	}
 
 	if (process->name) {
 		free(process->name);
 	}
-	process->name = strndup(deparenthesised_name, text_buffer_size);
+	process->name = strndup(procname, text_buffer_size);
 	process->rss *= getpagesize();
 
 	if (!cur->memmax) {
@@ -214,13 +232,85 @@ static int process_parse_stat(struct process *process)
 	return 0;
 }
 
+#ifdef IOSTATS
+static int process_parse_io(struct process *process)
+{
+	static const char *read_bytes_str="read_bytes:";
+	static const char *write_bytes_str="write_bytes:";
+
+	char line[BUFFER_LEN] = { 0 }, filename[BUFFER_LEN];
+	int ps;
+	int rc;
+	char *pos, *endpos;
+	unsigned long long read_bytes, write_bytes;
+
+	snprintf(filename, sizeof(filename), PROCFS_TEMPLATE_IO, process->pid);
+
+	ps = open(filename, O_RDONLY);
+	if (ps < 0) {
+		/* The process must have finished in the last few jiffies!
+		 * Or, the kernel doesn't support I/O accounting.
+		 */
+		return 1;
+	}
+
+	rc = read(ps, line, sizeof(line));
+	close(ps);
+	if (rc < 0) {
+		return 1;
+	}
+
+	pos = strstr(line, read_bytes_str);
+	if (pos == NULL) {
+		/* these should not happen (unless the format of the file changes) */
+		return 1;
+	}
+	pos += strlen(read_bytes_str);
+	process->read_bytes = strtoull(pos, &endpos, 10);
+	if (endpos == pos) {
+		return 1;
+	}
+
+	pos = strstr(line, write_bytes_str);
+	if (pos == NULL) {
+		return 1;
+	}
+	pos += strlen(write_bytes_str);
+	process->write_bytes = strtoull(pos, &endpos, 10);
+	if (endpos == pos) {
+		return 1;
+	}
+
+	if (process->previous_read_bytes == ULLONG_MAX) {
+		process->previous_read_bytes = process->read_bytes;
+	}
+	if (process->previous_write_bytes == ULLONG_MAX) {
+		process->previous_write_bytes = process->write_bytes;
+	}
+
+	/* store the difference of the byte counts */
+	read_bytes = process->read_bytes - process->previous_read_bytes;
+	write_bytes = process->write_bytes - process->previous_write_bytes;
+
+	/* backup the counts for next time around */
+	process->previous_read_bytes = process->read_bytes;
+	process->previous_write_bytes = process->write_bytes;
+
+	/* store only the difference here... */
+	process->read_bytes = read_bytes;
+	process->write_bytes = write_bytes;
+
+	return 0;
+}
+#endif
+
 /******************************************
  * Get process structure for process pid  *
  ******************************************/
 
 /* This function seems to hog all of the CPU time.
  * I can't figure out why - it doesn't do much. */
-static int calculate_cpu(struct process *process)
+static int calculate_stats(struct process *process)
 {
 	int rc;
 
@@ -230,6 +320,12 @@ static int calculate_cpu(struct process *process)
 		return 1;
 	/* rc = process_parse_statm(process); if (rc) return 1; */
 
+#ifdef IOSTATS
+	rc = process_parse_io(process);
+	if (rc)
+		return 1;
+#endif
+
 	/*
 	 * Check name against the exclusion list
 	 */
@@ -274,7 +370,7 @@ static int update_process_table(void)
 			}
 
 			/* compute each process cpu usage */
-			calculate_cpu(p);
+			calculate_stats(p);
 		}
 	}
 
@@ -395,18 +491,34 @@ inline static void calc_cpu_each(unsigned long long total)
 	}
 }
 
+#ifdef IOSTATS
+static void calc_io_each(void)
+{
+	struct process *p;
+	unsigned long long sum = 0;
+	
+	for (p = first_process; p; p = p->next)
+		sum += p->read_bytes + p->write_bytes;
+
+	if(sum == 0)
+		sum = 1; /* to avoid having NANs if no I/O occured */
+	for (p = first_process; p; p = p->next)
+		p->io_perc = 100.0 * (p->read_bytes + p->write_bytes) / (float) sum;
+}
+#endif
+
 /******************************************
  * Find the top processes				  *
  ******************************************/
 
 /* free a sp_process structure */
-void free_sp(struct sorted_process *sp)
+static void free_sp(struct sorted_process *sp)
 {
 	free(sp);
 }
 
 /* create a new sp_process structure */
-struct sorted_process *malloc_sp(struct process *proc)
+static struct sorted_process *malloc_sp(struct process *proc)
 {
 	struct sorted_process *sp;
 	sp = malloc(sizeof(struct sorted_process));
@@ -417,7 +529,7 @@ struct sorted_process *malloc_sp(struct process *proc)
 }
 
 /* cpu comparison function for insert_sp_element */
-int compare_cpu(struct process *a, struct process *b)
+static int compare_cpu(struct process *a, struct process *b)
 {
 	if (a->amount < b->amount) {
 		return 1;
@@ -429,7 +541,7 @@ int compare_cpu(struct process *a, struct process *b)
 }
 
 /* mem comparison function for insert_sp_element */
-int compare_mem(struct process *a, struct process *b)
+static int compare_mem(struct process *a, struct process *b)
 {
 	if (a->totalmem < b->totalmem) {
 		return 1;
@@ -440,9 +552,29 @@ int compare_mem(struct process *a, struct process *b)
 	}
 }
 
+/* CPU time comparision function for insert_sp_element */
+static int compare_time(struct process *a, struct process *b)
+{
+	return b->total_cpu_time - a->total_cpu_time;
+}
+
+#ifdef IOSTATS
+/* I/O comparision function for insert_sp_element */
+static int compare_io(struct process *a, struct process *b)
+{
+	if (a->io_perc < b->io_perc) {
+		return 1;
+	} else if (a->io_perc > b->io_perc) {
+		return -1;
+	} else {
+		return 0;
+	}
+}
+#endif
+
 /* insert this process into the list in a sorted fashion,
  * or destroy it if it doesn't fit on the list */
-int insert_sp_element(struct sorted_process *sp_cur,
+static int insert_sp_element(struct sorted_process *sp_cur,
 		struct sorted_process **p_sp_head, struct sorted_process **p_sp_tail,
 		int max_elements, int compare_funct(struct process *, struct process *))
 {
@@ -498,7 +630,7 @@ int insert_sp_element(struct sorted_process *sp_cur,
 }
 
 /* copy the procs in the sorted list to the array, and destroy the list */
-void sp_acopy(struct sorted_process *sp_head, struct process **ar, int max_size)
+static void sp_acopy(struct sorted_process *sp_head, struct process **ar, int max_size)
 {
 	struct sorted_process *sp_cur, *sp_tmp;
 	int x;
@@ -517,14 +649,27 @@ void sp_acopy(struct sorted_process *sp_head, struct process **ar, int max_size)
  * Results are stored in the cpu,mem arrays in decreasing order[0-9]. *
  * ****************************************************************** */
 
-void process_find_top(struct process **cpu, struct process **mem)
+void process_find_top(struct process **cpu, struct process **mem,
+		struct process **ptime
+#ifdef IOSTATS
+		, struct process **io
+#endif
+		)
 {
 	struct sorted_process *spc_head = NULL, *spc_tail = NULL, *spc_cur = NULL;
 	struct sorted_process *spm_head = NULL, *spm_tail = NULL, *spm_cur = NULL;
+	struct sorted_process *spt_head = NULL, *spt_tail = NULL, *spt_cur = NULL;
+#ifdef IOSTATS
+	struct sorted_process *spi_head = NULL, *spi_tail = NULL, *spi_cur = NULL;
+#endif
 	struct process *cur_proc = NULL;
 	unsigned long long total = 0;
 
-	if (!top_cpu && !top_mem) {
+	if (!top_cpu && !top_mem && !top_time
+#ifdef IOSTATS
+			&& !top_io
+#endif
+	   ) {
 		return;
 	}
 
@@ -532,6 +677,9 @@ void process_find_top(struct process **cpu, struct process **mem)
 	update_process_table();		/* update the table with process list */
 	calc_cpu_each(total);		/* and then the percentage for each task */
 	process_cleanup();			/* cleanup list from exited processes */
+#ifdef IOSTATS
+	calc_io_each();			/* percentage of I/O for each task */
+#endif
 
 	cur_proc = first_process;
 
@@ -546,8 +694,29 @@ void process_find_top(struct process **cpu, struct process **mem)
 			insert_sp_element(spm_cur, &spm_head, &spm_tail, MAX_SP,
 				&compare_mem);
 		}
+		if (top_time) {
+			spt_cur = malloc_sp(cur_proc);
+			insert_sp_element(spt_cur, &spt_head, &spt_tail, MAX_SP,
+				&compare_time);
+		}
+#ifdef IOSTATS
+		if (top_io) {
+			spi_cur = malloc_sp(cur_proc);
+			insert_sp_element(spi_cur, &spi_head, &spi_tail, MAX_SP,
+				&compare_io);
+		}
+#endif
 		cur_proc = cur_proc->next;
 	}
-	sp_acopy(spc_head, cpu, MAX_SP);
-	sp_acopy(spm_head, mem, MAX_SP);
+
+	if (top_cpu)
+		sp_acopy(spc_head, cpu, MAX_SP);
+	if (top_mem)
+		sp_acopy(spm_head, mem, MAX_SP);
+	if (top_time)
+		sp_acopy(spt_head, ptime, MAX_SP);
+#ifdef IOSTATS
+	if (top_io)
+		sp_acopy(spi_head, io,MAX_SP);
+#endif
 }
