@@ -1,3 +1,4 @@
+#ifdef X11
 #ifndef X11_H_
 #define X11_H_
 
@@ -19,6 +20,7 @@
 enum _window_type {
 	TYPE_NORMAL = 0,
 	TYPE_DOCK,
+	TYPE_PANEL,
 	TYPE_DESKTOP,
 	TYPE_OVERRIDE
 };
@@ -39,7 +41,11 @@ enum _window_hints {
 struct conky_window {
 	Window root, window, desktop;
 	Drawable drawable;
+	Visual *visual;
+	Colormap colourmap;
 	GC gc;
+	long border_inner_margin, border_outer_margin, border_width;
+
 #ifdef HAVE_XDBE
 	XdbeBackBuffer back_buffer;
 #endif
@@ -76,11 +82,15 @@ extern int workarea[4];
 
 extern struct conky_window window;
 
-void init_X11(void);
+void init_X11(const char*);
 void init_window(int use_own_window, int width, int height, int set_trans,
 	int back_colour, char **argv, int argc);
+void destroy_window(void);
 void create_gc(void);
-void set_transparent_background(Window win);
+void set_transparent_background(void);
 long get_x11_color(const char *);
+void get_x11_desktop_info(Display *display, Atom atom);
+void set_struts(int);
 
 #endif /*X11_H_*/
+#endif /* X11 */
